/*
 * ConstraintsList.cpp
 *
 *  Created on: Jul 8, 2015
 *      Author: Andre Dozier
 */

#ifndef CONSTRAINTSLIST_H_
#define CONSTRAINTSLIST_H_

#include <vector>
#include <stdexcept>
#include <cmath>
#include "Constraints.h"
#include "ConstraintParser.h"
using namespace std;

class ConstraintsList {

private:

	vector<ConstraintParser> _decisionParsers;
	vector<ConstraintParser> _stateParsers;
	vector<Constraints> _decisionConstraints;
	vector<Constraints> _stateConstraints;

public:

	void addDecisionParser(ConstraintParser& parser) {
		_decisionParsers.push_back(parser);
	}

	void addStateParser(ConstraintParser& parser) {
		_stateParsers.push_back(parser);
	}

	int dcSize() {
		return _decisionConstraints.size();
	}

	int dpSize() {
		return _decisionParsers.size();
	}

	int scSize() {
		return _stateConstraints.size();
	}

	int spSize() {
		return _stateParsers.size();
	}

	void sizeCheck(int nTimesteps, int size) {
		if (size <= 0)
			throw runtime_error("Must provide at least one constraint parser.");

		if (size != 1 && size != nTimesteps)
			throw runtime_error("Must provide 1 or nTimesteps constraint parsers.");
	}

	void check(int nTimesteps, LinearGrids states, LinearGrids decisions) {

		// size checks
		sizeCheck(nTimesteps, dpSize());
		sizeCheck(nTimesteps, spSize());

		// build constraints
		int t;
		for (t = 0; t < states.size(); t++) {
			Constraints sc = _stateParsers[t].getConstraints(states[t], decisions[t]);
			_stateConstraints.push_back(sc);
		}
		for (t = 0; t < states.size(); t++) {
			Constraints dc = _decisionParsers[t].getConstraints(states[t], decisions[t]);
			_decisionConstraints.push_back(dc);
		}
	}

	bool satisfied(int t, long long state_i, long long dec_i) {

		// check states
		switch (scSize()) {

		case 0:
			break;

		case 1:
			if (!_stateConstraints[0].satisfied(state_i, dec_i))
				return false;
			break;

		default:
			if (!_stateConstraints[t].satisfied(state_i, dec_i))
				return false;
			break;
		}

		// check decisions
		switch (dcSize()) {

		case 0:
			break;

		case 1:
			if (!_decisionConstraints[0].satisfied(dec_i, dec_i))
				return false;
			break;

		default:
			if (!_decisionConstraints[t].satisfied(dec_i, dec_i))
				return false;
			break;
		}

		return true;
	}
};

#endif /* ConstraintsList_H_ */



