/*
 * DyproModel.h
 *
 *  Created on: Feb 2, 2015
 *      Author: Andre Dozier
 */

#ifndef DyproModel_H_
#define DyproModel_H_

// mapping
#include <vector>
#include <map>

// io and strings
#include <cstring>
#include <string>
#include <iostream>
#include <iomanip>
#include <fstream>

// math
#include <cfloat>
#include <numeric>
#include <cstdio>
#include <algorithm>
#include <cmath>

// exceptions
#include <stdexcept>

// timing
#include <ctime>

// xml project
#include "rapidxml.hpp"

// within project
#include "BoundsCheck.h"
#include "LinearGrid.h"
#include "LinearGrids.h"
#include "CartesianProduct.h"
#include "SequentialDecisionProcess.h"
#include "DyproSolutionPoint.h"
#include "ProbabilityWeightedSet.h"
#include "DyproUtils.h"
#include "ObjectiveAccumulation.h"
#include "ObjectiveDirection.h"
#include "ObjectiveTie.h"
#include "DyproSolution.h"
#include "DyproTrace.h"
#include "StopWatch.h"
#include "ConstraintsList.h"

// std
using namespace std;
using namespace rapidxml;

// time check
const int TIMECHECK = 30; // seconds

class DyproModel {

private:

	// solve
	void _expectedValue(SequentialDecisionProcess&, int, long long, long long,
			DyproSolutionAtTime&, Bounds&, Bounds&);
	void _bestDecision(DyproSolutionPoint&, SequentialDecisionProcess&, int,
			long long, DyproSolutionAtTime&, Bounds&, Bounds&);
	void _evalStates(DyproSolutionAtTime&, SequentialDecisionProcess&, int,
			DyproSolutionAtTime&);
	void _checkTotalTime(double, double, int);

	// trace
	vector<DyproTrace> _traces;
	void _trace(DyproTrace& currTrace, SequentialDecisionProcess& sdp);

	// time
	bool _checked;
	vector<vector<double> > _secondsPerEval;
	StopWatch _sw;

public:

	// objective type
	ObjectiveAccumulator* objAccum;
	ObjectiveDirector* objDir;
	ObjectiveTieBreaker* objTie;

	// states and decisions spaces
	LinearGrids states;
	LinearGrids decisions;

	// constraints
	ConstraintsList constraints;

	// numbers of timesteps, iterations, variables and traces
	unsigned int nTimesteps;
	unsigned int nIterations;
	unsigned int nRandomVars;
	unsigned int nTraces;
	unsigned int nTraceTimesteps;

	// random realizations
	vector<ProbabilityWeightedSet> probSet;
	double acceptableRisk_LowState;
	double acceptableRisk_HighState;
	double acceptableRisk_InfValue;

	// iterations
	double errorTarget;
	vector<double> maxErrors;

	// dynamic features
	double discount;

	// solution
	DyproSolution solution;

	// trace
	bool snapTrace;
	vector<double> initialState;
	vector<RandomTimeseries> rndVarTimeseries;

	// final value
	double salvageValue;

	// verbosity
	bool verbose;

	// constructors
	DyproModel(ObjectiveAccumulation, ObjectiveDirection, ObjectiveTie,
			LinearGrids&, LinearGrids&, unsigned int, ConstraintsList&);

	// properties
	int numStateVars(int);
	int numDecisionVars(int);
	Bounds getDecisionBounds(int);
	Bounds getStateBounds(int);

	// reference properties
	ProbabilityWeightedSet& getProbSet(int t);
	void getRandomTimeseries(RandomTimeseries& s, int t);
	DyproSolution& getSolution();
	vector<DyproTrace>& getTraces();

	// save solutions, errors, and traces
	void loadSolution(istream& fs);
	void saveSolution(ostream& o, int maxPrints = 0);
	void saveErrors(ostream& o, int maxPrints = 0);
	void saveTraces(ostream& o, int maxPrints = 0);

	// reseed
	void reseed();

	// solve the dynamic program
	void solve(SequentialDecisionProcess& sdp, bool save_solution = false,
			const char* solution_file = "solution.txt", const char* error_file =
					"errors.txt");

	// trace the optimal solution
	void addRandomTimeseries(RandomTimeseries& ts);
	void trace(SequentialDecisionProcess& sdp);

	// solve and trace
	void execute(SequentialDecisionProcess& sdp, bool just_trace = false,
			bool save_output = false, bool save_intermediate = false,
			bool load_prev_solution = false, const char* solution_file =
					"solution.txt", const char* error_file = "errors.txt",
			const char* trace_file = "traces.txt");

	// run check
	void runChecks();

	// build a DyproModel
	static DyproModel fromString(char *);
	static DyproModel fromFile(const char *);
};

#endif /* DyproModel_H_ */
